<?php

/*
This is a "debug" API. It is very simple yet very versatile. Just include this 
file, enable debugging and optionally set some options, and the strategically 
place calls to the debug() function in and around the code you want to debug.
The debug() function takes anything as an argument which will be displayed 
at the end of the HTML output in a list. This output can be customised.

See the file "debug.txt" for more information.
*/


/* debuglog - set the errorlog level */

function debuglog($log) {

	_debug('log',$log);
	if ($log == 1)
		register_shutdown_function('_msglogdump');
}


/* debug - display message or add to message queue */

function debug($msg = NULL, $trunc = 0) {

	$log = _debug('log');

	if ($msg === NULL)
		return $log;

	if ($log == 0)
		return;

	list($file,$line,$func) = debug_caller(1);
	$func = debug_caller(2,'function');

	if (($t = _debug('file')) && $t != $file)
		return;
	if (($t = _debug('nofile')) && strstr($t,$file))
		return;

	if ($msg === TRUE)
		$msg = '(true)';
	else
	if ($msg === FALSE)
		$msg = '(false)';
	else
	if (is_resource($msg))
		$msg = 'Resource: '.get_resource_type($msg);
	else
	if (is_object($msg))
		$msg = 'Object: '.get_class($msg);
	else
	if (is_array($msg))
		$msg = 'Array: {'. _flatten($msg) .'}';

	if ($trunc == 0 && ($t = _debug('trunc')))
		$trunc = $t;

	if ($trunc && $trunc != -1 && strlen($msg) > $trunc)
		$msg = substr($msg,0,$trunc).' ...';

	$msg = htmlentities($msg);

	if (_debug('nl'))
		$msg = str_replace("\n",'\n',$msg);

	$c = '';
	if (_debug('caller')) {
		list($f,$l,$na) = debug_caller(2);
		$c = "($f,$l)";
	}

	if (!_debug('nofunc'))
		$msg = "$c($file,$line,$func) $msg";
	else
		$msg = "$c($file,$line) $msg";

	if ($log == -1)
		print $msg . '<br>';
	else
	if ($log == 100)
		print '<!-- '.$msg.' -->'.PHP_EOL;
	else
	if ($log == 200)
		$GLOBALS['debug'] .= $msg.'<br>'.PHP_EOL;
	else
		_debug('addmsg',$msg);
}


/* _msglogdump - display the message log (as a shutdown fuction) */

function _msglogdump() {

	$log = _debug('log');

	if ($log <= 0 || $log >= 100)		// special cases
		return;

	print _debug('open');
	foreach (_debug('msgs') as $msg)
		print "\n$msg";
	print _debug('close');
}


/* debug handler configuration */

function _debug($key = NULL, $val = NULL) {
static $debug = array();

	if ($debug == array()) {

		$GLOBALS['debug'] = '';

		$debug['log'] = 0;
		$debug['msgs'][] = '<b>message log:</b>';
		$debug['open'] = "\n<div style='background: white; color: black;'><pre>";
		$debug['close'] = "</pre></div>";

		// default configuration:

		$debug['caller'] = 1;
		$debug['nl'] = 1;
	}

	if ($key === NULL)
		return $debug;

	if ($key == 'addmsg')				// special case
		return $debug['msgs'][] = $val;

	if ($val !== NULL)
		return $debug[$key] = $val;

	if (isset($debug[$key]))			// config value
		return $debug[$key];

	return '';
}


/* _flatten - an implode() that handles associative arrays */

function _flatten($a) {

	if (isset($a[0]) && !is_array($a[0]))
		return implode(',',$a);

	$s = '';
	foreach ($a as $k => $v) {
		if (is_array($v))
			$s .= "$k="._flatten($v);
		else
			$s .= "$k='$v',";
	}

	return rtrim($s,',');
}

/* debug_caller - get caller of caller (why is this function so big?) */

function debug_caller($depth = 1, $elem = NULL, $nb = 0, $rbt = 0) {

	if (defined('DEBUG_BACKTRACE_IGNORE_ARGS'))
		$bt = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
	else
		$bt = debug_backtrace();

	if ($rbt)
		return $bt;

	if ($elem) {
		$s = (isset($bt[$depth][$elem])) ? $bt[$depth][$elem] : '';
		if (!$nb && $elem == 'file')
			$s = basename($s);
		return $s;
	}

	if (!isset($bt[$depth]))
		return(array('?',$depth,''));

	$bt = $bt[$depth];
	if ($nb) 
		$file = (isset($bt['file'])) ? $bt['file'] : '';
	else
		$file = (isset($bt['file'])) ? basename($bt['file']) : '';
	$line = (isset($bt['line'])) ? $bt['line'] : '';
	$func = $bt['function'];

	if ($elem === '')
		return "$file,$line,$func";

	return array($file,$line,$func);
}

